# coding: utf-8
"""
Placeholder
"""
from django.db import models
from django.contrib.auth.models import User, Group, Permission
from django.db.models.query import Q
from django.utils.translation import ugettext as _

from jmb.core.middleware.thread_local import get_current_user


class DomainManager(models.Manager):
    def get_queryset(self):

        self.user = get_current_user()
        original = super(DomainManager, self).get_queryset()

        if self.user and self.user.is_authenticated() and self.user.has_perm('mailserver.supermaster_perm'):
            return original.order_by('name')
        elif self.user and self.user.is_authenticated() and self.user.has_perm('mailserver.postmaster_perm'):
            return original.filter(postmaster=self.user).order_by('name')
        else:
            return super(DomainManager, self).none()

class AccountManager(models.Manager):
    def get_queryset(self):

        self.user = get_current_user()
        original = super(AccountManager, self).get_queryset()

        if self.user and self.user.is_authenticated() and self.user.has_perm('mailserver.supermaster_perm'):
            return original.order_by('account')
        elif self.user and self.user.is_authenticated() and self.user.has_perm('mailserver.postmaster_perm'):
            return original.filter(domain__postmaster=self.user).order_by('account')
        else:
            return super(AccountManager, self).none()
        
class AliasManager(models.Manager):
    def get_queryset(self):

        self.user = get_current_user()
        original = super(AliasManager, self).get_queryset() 

        if self.user and self.user.is_authenticated() and self.user.has_perm('mailserver.supermaster_perm'):
            return original.order_by('alias')
        elif self.user and self.user.is_authenticated() and self.user.has_perm('mailserver.postmaster_perm'):
            return original.filter(domain__postmaster=self.user).order_by('alias')
        else:
            return super(AliasManager, self).none()

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

############################
# DOMINIO                  #
############################
class Domain (models.Model):
    objects = DomainManager()
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    creator = models.ForeignKey (
        User, default=get_current_user,
        related_name="creator",
        verbose_name=_("Created by"),
    )
    postmaster = models.ManyToManyField (
        User,
        blank=True,
        verbose_name="Postmaster",
    )
    ## Campi
    name = models.CharField (
        max_length=127,
        unique=True,
        verbose_name=_("Domain"),
    )
    description = models.TextField (
        blank=True,
        verbose_name=_("Descrizione"),
    )
    users = models.SmallIntegerField (
        null=True, blank=True,
        verbose_name=_("Max Utenti"),
    )
    # aliases = models.SmallIntegerField (
    #     null=True, blank=True,
    #     verbose_name=_("Max Alias"),
    # )
    quota  = models.IntegerField(
        null=True, blank=True,
        verbose_name="Max Quota",
        help_text = 'In Megabytes (MB)'
    )
    antispam = models.BooleanField (
        null=False, blank=True, default=False,
        verbose_name="Antispam",
    )
    disable_imap_login = models.BooleanField (
        verbose_name=_("Disable IMAP"),  default=True,
    )
    disable_pop_login = models.BooleanField (
        verbose_name=_("Disable POP"),  default=True,
    )
    transport = models.CharField(
        blank=True,
        max_length=255,
        verbose_name='Transport',
        help_text = _('If an IP is present all email for this domain will be forwarded to this IP')

    )
    domain_transport_wins = models.BooleanField(default=True, null=False)
    check_relay_users = models.BooleanField(default=True, null=False,
                 help_text=_("If relay is enable (transport != virtual), check user's existance"))
    
    class Meta:
        verbose_name = "Dominio Mail"
        verbose_name_plural = "Domini Mail"
        permissions = (
            ('supermaster_perm', 'complete mailserver administration'),
            ('postmaster_perm', 'mailserver administration related to his domains'),
        )

    def __unicode__(self):
        return unicode(self.name)
    
#################################
# ACCOUNT                       #
#################################
class Account (models.Model):
    objects = AccountManager()
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    domain = models.ForeignKey (
        "Domain",
        verbose_name="dominio",
        related_name="accounts",
#        limit_choices_to={'postmaster' : get_current_user()}
    )
    postmaster = models.ForeignKey (
        User,
        related_name="postmaster",
        verbose_name="Creato da",
    )
    user = models.ForeignKey (
        User,
        blank=True, null=True,
        verbose_name="User",
    )
    ## Campi
    account = models.CharField (
        max_length=127,
        verbose_name="Account",
    )
    pwd = models.CharField (
        max_length=127,
        verbose_name="Password",
    )
    quota = models.IntegerField (
        verbose_name="Quota",
        help_text = 'In Megabytes (MB)'
    )
    home_dir = models.CharField (
        max_length=255,
        null=True, blank=True,
        verbose_name="Path",
    )
    backup = models.BooleanField (
        null=False, blank=True, default=False,
        verbose_name="Backup",
    )
    antispam = models.BooleanField (
        null=False, blank=True, default=False,
        verbose_name="Antispam",
    )
    disable_imap_login = models.BooleanField (
        verbose_name="Disable IMAP", default=True,
    )
    disable_pop_login = models.BooleanField (
        verbose_name="Disable POP",  default=True,
    )
    transport = models.CharField(max_length=50, null=True, blank=True,
        help_text = _('If an IP is present all email for this domain will be forwarded to this IP')
    )
    def full_name(self):
        return "%s@%s" % (self.account, self.domain)
    
    def set_password(self, raw_password):
        from random import randint
        from crypt import crypt
        import string

        salt_chars = './' + string.ascii_letters + string.digits
        salt = salt_chars[randint(0, len(salt_chars))] + salt_chars[randint(0, len(string.digits))]

        return crypt(raw_password, salt)

    def save(self):
        if not self.id:
            self.pwd = self.set_password(self.pwd)
            self.home_dir = "%s/%s/" % (self.domain, self.account)
            # antispam is active if it is active for the domain
            if self.domain.antispam:
                self.antispam = self.domain.antispam

        return super(Account, self).save()

    class Meta:
        verbose_name = "Account Mail"
        verbose_name_plural = "Account Mail"
        unique_together = (("account", "domain"),)

    def __unicode__(self):
        return u'%s@%s' % (self.account, self.domain.name)
    
###########################
# ALIAS                   #
###########################
class Alias (models.Model):
    objects = AliasManager()
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    domain = models.ForeignKey (
        Domain,
        verbose_name="dominio",
        related_name="aliases",
#        limit_choices_to={'postmaster' : get_current_user()}
    )
    alias = models.CharField (
        max_length=127,
        help_text = "Simple name without '@' simbol"
    )
    address = models.CharField (
        max_length=127,
        verbose_name="Email",
        help_text = "A list of names or addresses. " + \
            "Simple names are searched for in this domain"
    )

    class Meta:
        verbose_name = "Alias"
        verbose_name_plural = "Aliases"
        unique_together = (("alias", "address", "domain"),)

    def __unicode__(self):
        return u'%s@%s' % (self.alias, self.domain.name)
    
################################
# VACATION                     #
################################
STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

class Vacation (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    account = models.ForeignKey (
        "Account",
        verbose_name="Account",
    )
    ## Campi
    date_start = models.DateField (
        verbose_name="Data Inizio",
    )
    date_end = models.DateField (
        verbose_name="Data Fine",
    )
    vacation = models.TextField (
        blank = True,
        verbose_name="Vacation",
    )

    class Meta:
        verbose_name = "Vacation"
        verbose_name_plural = "Vacation"       

    def __unicode__(self):
        return unicode("%s - %s" % (self.account.account, self.vacation))
    
def add_group():
    from jungle.functions.tools import add_group_with_app_permissions
    
    add_group_with_app_permissions('Postmaster', Domain._meta.app_label)
    add_group_with_app_permissions('Supermaster', Domain._meta.app_label)

    supermaster = Group.objects.get(name__exact = 'Supermaster')
    try:
        supermaster.permissions.add(Permission.objects.get(codename__exact='supermaster_perm'))
        supermaster.save()
    except:
        pass

    postmaster = Group.objects.get(name__exact = 'Postmaster')
    try:
        postmaster.permissions.add(Permission.objects.get(codename__exact='postmaster_perm'))
        postmaster.save()
    except:
        pass

